@@ -111,6 +111,7 @@
 #include "gtksourcecompletionprovider.h"
 #include "gtksourcecompletioncontainer.h"
 #include "gtksourcebuffer.h"
+#include "gtksource-marshal.h"
 #include "gtksourceview.h"
 
 enum
@@ -742,21 +743,25 @@ gtk_source_completion_activate_proposal (GtkSourceCompletion *completion)
 static void
 update_info_position (GtkSourceCompletion *completion)
 {
-	GdkScreen *screen;
+	GdkDisplay *display;
+	GdkMonitor *monitor;
+	GdkWindow *window;
+	GdkRectangle geom;
 	gint x, y;
 	gint width, height;
-	gint screen_width;
 	gint info_width;
 
 	gtk_window_get_position (GTK_WINDOW (completion->priv->main_window), &x, &y);
 	gtk_window_get_size (GTK_WINDOW (completion->priv->main_window), &width, &height);
 	gtk_window_get_size (GTK_WINDOW (completion->priv->info_window), &info_width, NULL);
 
-	screen = gtk_window_get_screen (GTK_WINDOW (completion->priv->main_window));
-	screen_width = gdk_screen_get_width (screen);
+	display = gtk_widget_get_display (GTK_WIDGET (completion->priv->main_window));
+	window = gtk_widget_get_window (GTK_WIDGET (completion->priv->main_window));
+	monitor = gdk_display_get_monitor_at_window (display, window);
+	gdk_monitor_get_geometry (monitor, &geom);
 
 	/* Determine on which side to place it */
-	if (x + width + info_width >= screen_width)
+	if (x + width + info_width >= geom.width)
 	{
 		x -= info_width;
 	}
@@ -2461,8 +2466,12 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GtkSourceCompletionClass, show),
-			      NULL, NULL, NULL,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	g_signal_set_va_marshaller (signals[SHOW],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            g_cclosure_marshal_VOID__VOIDv);
 
 
 	/**
@@ -2477,8 +2486,12 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GtkSourceCompletionClass, hide),
-			      NULL, NULL, NULL,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	g_signal_set_va_marshaller (signals[HIDE],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            g_cclosure_marshal_VOID__VOIDv);
 
 	/**
 	 * GtkSourceCompletion::populate-context:
@@ -2493,10 +2506,14 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		              G_STRUCT_OFFSET (GtkSourceCompletionClass, populate_context),
-		              NULL, NULL, NULL,
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__OBJECT,
 		              G_TYPE_NONE,
 		              1,
 		              GTK_SOURCE_TYPE_COMPLETION_CONTEXT);
+	g_signal_set_va_marshaller (signals[POPULATE_CONTEXT],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            g_cclosure_marshal_VOID__OBJECTv);
 
 	/* Actions */
 
@@ -2528,11 +2545,15 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GtkSourceCompletionClass, move_cursor),
-			      NULL, NULL, NULL,
+			      NULL, NULL,
+			      _gtk_source_marshal_VOID__ENUM_INT,
 			      G_TYPE_NONE,
 			      2,
 			      GTK_TYPE_SCROLL_STEP,
 			      G_TYPE_INT);
+	g_signal_set_va_marshaller (signals [MOVE_CURSOR],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            _gtk_source_marshal_VOID__ENUM_INTv);
 
 	/**
 	 * GtkSourceCompletion::move-page:
@@ -2565,11 +2586,15 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GtkSourceCompletionClass, move_page),
-			      NULL, NULL, NULL,
+			      NULL, NULL,
+			      _gtk_source_marshal_VOID__ENUM_INT,
 			      G_TYPE_NONE,
 			      2,
 			      GTK_TYPE_SCROLL_STEP,
 			      G_TYPE_INT);
+	g_signal_set_va_marshaller (signals [MOVE_PAGE],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            _gtk_source_marshal_VOID__ENUM_INTv);
 
 	/**
 	 * GtkSourceCompletion::activate-proposal:
@@ -2588,8 +2613,12 @@ gtk_source_completion_class_init (GtkSourceCompletionClass *klass)
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GtkSourceCompletionClass, activate_proposal),
-			      NULL, NULL, NULL,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	g_signal_set_va_marshaller (signals [ACTIVATE_PROPOSAL],
+	                            G_TYPE_FROM_CLASS (klass),
+	                            g_cclosure_marshal_VOID__VOIDv);
 
 	/* Key bindings */
 	binding_set = gtk_binding_set_by_class (klass);
@@ -2953,7 +2982,7 @@ gtk_source_completion_remove_provider (GtkSourceCompletion          *completion,
 		return FALSE;
 	}
 
-	completion->priv->providers = g_list_remove_link (completion->priv->providers, item);
+	completion->priv->providers = g_list_delete_link (completion->priv->providers, item);
 
 	g_object_unref (provider);
 
